from collections import namedtuple
from getpass import getpass
import os

from .grammarbot import HemoglobinGrammarBot as GrammarBotClient
from .files import HemoglobinFile

BaseConfig = namedtuple("Config", ["apikey", "paths", "language"])


class Config(BaseConfig):
    GRAMMARBOT_CLIENT_APIKEY_ENV_NAME = "GRAMMARBOT_CLIENT_APIKEY"
    GRAMMARBOT_LANGUAGE_ENV_NAME = "GRAMMARBOT_LANGUAGE"

    @classmethod
    def from_args(cls, args):
        apikey = args.apikey
        if not apikey:
            if cls.GRAMMARBOT_CLIENT_APIKEY_ENV_NAME in os.environ:
                apikey = os.environ[cls.GRAMMARBOT_CLIENT_APIKEY_ENV_NAME]
            else:
                apikey = getpass("Enter your API key: ")
                if not apikey:
                    print("GrammarBot API key is not defined.")
                    exit(1)
        language = args.language
        if not language:
            if cls.GRAMMARBOT_LANGUAGE_ENV_NAME in os.environ:
                language = os.environ[cls.GRAMMARBOT_LANGUAGE_ENV_NAME]
        return cls(apikey=apikey, paths=args.path, language=args.language)


class Hemoglobin(object):
    GRAMMARBOT_CLIENT = GrammarBotClient
    HEMOGLOBIN_FILE = HemoglobinFile

    ACCEPTABLE_FILE_EXTENSIONS = ["txt", "md"]

    @classmethod
    def from_config(cls, config):
        return cls(apikey=config.apikey, paths=config.paths, language=config.language)

    def __init__(self, apikey, paths, json=False, language="en_US"):
        self.apikey = apikey
        self.paths = paths
        self.language = language

        self._grammarbot = None
        self._files = []

    @property
    def grammarbot(self):
        if not self._grammarbot:
            self._grammarbot = self.GRAMMARBOT_CLIENT(
                api_key=self.apikey, language=self.language
            )
        return self._grammarbot

    @property
    def files(self):
        """A list of all the files to scan, based upon initial root path.
        Only lists files that have the right file extension.
        Wraps those files in to the TextStatFile object.
        """
        if not self._files:
            self._paths_walk()
        return self._files

    def _paths_walk(self):
        for path in self.paths:
            if os.path.isfile(path):
                self._claim_file(path)
            else:
                for root_path, _, file_names, _ in os.fwalk(path):
                    for file_name in file_names:
                        file_path = os.path.join(root_path, file_name)
                        self._claim_file(file_path)

    def _claim_file(self, file_path):
        """Takes file path, does some checks, and adds it to the files list if
        it meets the criteria.

        :param file_path: The path to the file to check
        :type file_path: str
        """
        if self._extension_ok(file_path):
            self._add_file(file_path)

    def _extension_ok(self, file_name):
        """Pass in a file path to see if the file is OK to be processed.

        :param file_path: Path of file to be checked
        :type file_path: str

        :return: True if this file is OK to process. False if not and should be
            rejected.
        :rtype: bool
        """
        # get the file extension, or the thing after the last dot
        # in the file name, to see if it's on the approved list
        extension = file_name.rsplit(".", maxsplit=1)[-1].lower()
        return extension in self.ACCEPTABLE_FILE_EXTENSIONS

    def _add_file(self, file_path):
        """Wraps file in processor object and adds to list of local variables

        :param file_path: Path of file to add
        :type file_path: str
        """
        self._files.append(self.HEMOGLOBIN_FILE.from_path(file_path, self))

    def to_dict(self):
        return dict(
            (grammarbot_file.f.name, grammarbot_file.to_dict())
            for grammarbot_file in self.files
        )



class HemoglobinFile(object):
    """An object that represents files on your system to run the tests against."""

    OPEN_MODE = "r"

    @classmethod
    def from_path(cls, file_path, cli):
        """Initialise this class when you don't have a file object, but do have
        the file path.

        :param file_path: Path to file to process
        :type file_path: str
        :param cli: parent config object
        """
        return cls(open(file_path, cls.OPEN_MODE), cli)

    def __init__(self, f, hemoglobin):
        """
        :param f: file object to run methods in this function over
        :param cli: parent CLI object to get config from
        """
        self.f = f
        self.hemoglobin = hemoglobin

        self._text = None
        self._response = None

    @property
    def text(self):
        """Get the contents of the file (memory inefficent)

        :return: Contents of file object this class was initialised with
        :rtype: str
        """
        if not self._text:
            self._text = self.f.read()
        return self._text

    @property
    def response(self):
        if not self._response:
            self._response = self.get_grammarbot_response()
        return self._response

    @property
    def matches(self):
        return self.response.matches

    def get_grammarbot_response(self):
        return self.hemoglobin.grammarbot.check(self.text)

    def to_dict(self):
        """For all of the tests/methods available in textstat, run through them
        all and produce the results as a dictionary.

        :return: A dictionary representation of the results
        :rtype: dict
        """
        return self.get_grammarbot_response().to_dict()



import argparse
from getpass import getpass
from json import dumps as json_dumps

from .hemoglobin import Hemoglobin, Config
from .grammarbot import Language as Languages


def create_args():
    """
    Creates arguments for this main function.
    Returns the created parser object.
    """
    parser = argparse.ArgumentParser(description="Check the contents of text files")

    parser.add_argument("apikey", help="Your API key for GrammarBot.", nargs="?")
    parser.add_argument(
        "--json",
        "-j",
        action="store_true",
        default=False,
        dest="use_json_output",
        help="Use argument to have the results output as json.",
    )
    parser.add_argument(
        "--language",
        "-l",
        help="Language to use.",
        choices=Languages,
        type=Languages,
        default=Languages.EN_US,
    )

    parser.add_argument("path", help="Where to find these files to parse.", nargs="+")
    return parser


def render_human(hemoglobin):
    for grammarbot_file in hemoglobin.files:
        print(grammarbot_file.f.name)
        for match in grammarbot_file.matches:
            print("\tSentence: {sentence}".format(sentence=match.sentence))
            print("\t\tMessage: {message}".format(message=match.message))
            print("\t\tPossible corrections:")
            for correction in match.corrections:
                print("\t\t\t{correction}".format(correction=correction))
            print("\t\tDetail:")
            print("\t\t\tType: {type}".format(type=match.type))
            print("\t\t\tCategory: {category}".format(category=match.category))
            print("\t\t\tRule: {rule}".format(rule=match.rule))
    print(
        "Number of API calls made: {api_calls_made}".format(
            api_calls_made=hemoglobin.grammarbot.api_calls_made
        )
    )


def render_json(hemoglobin):
    print(json_dumps(hemoglobin.to_dict()))


if __name__ == "__main__":
    args = create_args().parse_args()
    hemoglobin = Hemoglobin.from_config(Config.from_args(args))
    if args.use_json_output:
        render_json(hemoglobin)
    else:
        render_human(hemoglobin)



from enum import Enum
from typing import List

import mimeparse
import requests

from grammarbot import GrammarBotClient, GrammarBotApiResponse, GrammarBotMatch


class Language(Enum):
    EN_US = "en-US"
    EN_GB = "en-GB"


class GrammarBotException(Exception):
    pass


class HemoglobinGrammarBotMatch(GrammarBotMatch):
    @property
    def sentence(self) -> str:
        """
        Gives the rule category.
        """
        return self._match_json["sentence"]


class HemoglobinGrammarBotApiResponse(GrammarBotApiResponse):
    MATCH = HemoglobinGrammarBotMatch

    @property
    def matches(self) -> List[GrammarBotMatch]:
        """
        Different matches detected by the GrammarBot API.
        """
        return [self.MATCH(mjson) for mjson in self._json["matches"]]

    def to_dict(self):
        return self.matches


class HemoglobinGrammarBot(GrammarBotClient):
    API_RESPONSE = HemoglobinGrammarBotApiResponse
    MAX_CHARS = 10000

    def __init__(
        self,
        base_uri: str = "http://api.grammarbot.io/",
        api_key: str = "python-default",
        language: Language = Language.EN_US,
    ):
        super(HemoglobinGrammarBot, self).__init__(base_uri=base_uri, api_key=api_key)
        self.language = language
        self.api_calls_made = 0

    def _create_params(self, text):
        return {"language": self.language.value, "text": text, "api_key": self._api_key}

    def get_response(self, text: str):
        params = self._create_params(text)
        self.api_calls_made += 1
        return requests.get(self._endpoint, params=params)

    def check_response(self, response):
        main_mime_type, sub_mime_type, _ = mimeparse.parse_mime_type(
            response.headers["Content-Type"]
        )
        if not (main_mime_type == "application" and sub_mime_type == "json"):
            raise GrammarBotException(response.text)

    def check_under_max_chars(self, text):
        response = self.get_response(text)
        self.check_response(response)
        return self.API_RESPONSE(response.json())

    def check_over_max_chars(self, text):
        paras = text.splitlines()
        buffer = ""

        results = {"matches": []}

        for para in paras:
            if para:
                if len(para) > self.MAX_CHARS:
                    raise GrammarBotException(
                        "A paragraph is longer than {max_chars} maximum number of characters for GrammarBot. Processing cannot continue.".format(
                            max_chars=self.MAX_CHARS
                        )
                    )
                if (len(buffer) + len(para)) > self.MAX_CHARS:
                    response = self.get_response(buffer)
                    self.check_response(response)
                    results["matches"] += response.json()["matches"]
                    buffer = para
                else:
                    buffer += para

        response = self.get_response(buffer)
        self.check_response(response)
        results["matches"] += response.json()["matches"]

        return self.API_RESPONSE(results)

    def check(self, text: str):
        """
        Check a given piece of text for grammatical errors.
        :param text:
            Text to be checked using the API.
        """
        if len(text) < self.MAX_CHARS:
            return self.check_under_max_chars(text)
        else:
            return self.check_over_max_chars(text)



#!/usr/bin/env python

from distutils.core import setup

import hemoglobin

setup(
    name="hemoglobin",
    version=hemoglobin.__version__,
    description="A CLI wrapper for the grammarbot library",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Matt Copperwaite",
    author_email="matt@copperwaite.net",
    url="https://github.com/yamatt/python3-hemoglobin",
    packages=["hemoglobin"],
    scripts=["scripts/hemoglobin"],
    install_requires=open("requirements.txt", encoding="utf-8").readlines(),
    license="gplv3",
    project_urls={
        "Source": "https://github.com/yamatt/python3-hemoglobin",
        "Tracker": "https://github.com/yamatt/python3-hemoglobin/issues",
    },
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
        "Intended Audience :: Other Audience",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
